@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 2006 Tetsuya Kimata <kimata@acapulco.dyndns.org>
+ * Copyright (C) 2007 Tetsuya Kimata <kimata@acapulco.dyndns.org>
  *
  * All rights reserved.
  *
@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemIterator.cpp 2571 2007-10-12 23:13:36Z svn $
+ * $Id: UploadItemIterator.cpp 2756 2007-12-11 10:57:59Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -37,7 +37,7 @@
 #include "ReadLocker.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: UploadItemIterator.cpp 2571 2007-10-12 23:13:36Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItemIterator.cpp 2756 2007-12-11 10:57:59Z svn $");
 
 
 /******************************************************************************
@@ -47,7 +47,7 @@ UploadItemIterator::UploadItemIterator(apr_pool_t *pool,
                                        UploadItemManager *item_manager,
                                        apr_size_t start_index,
                                        apr_size_t end_index,
-                                       int(*cmp)(const void *a, const void *b))
+                                       int (*cmp)(const UploadItem *a, const UploadItem *b))
   : header_list_(NULL),
     curr_index_(0)
 {
@@ -99,7 +99,7 @@ void UploadItemIterator::reset()
  *****************************************************************************/
 void UploadItemIterator::init(apr_pool_t *pool, UploadItemList *item_list,
                               apr_size_t start_index, apr_size_t end_index,
-                              int(*cmp)(const void *a, const void *b))
+                              int (*cmp)(const UploadItem *a, const UploadItem *b))
 {
     apr_size_t list_size;
 
@@ -118,7 +118,8 @@ void UploadItemIterator::init(apr_pool_t *pool, UploadItemList *item_list,
     memcpy(header_list_, item_list->get_by_index(0),
            sizeof(UploadItem::header_t) * list_size);
 
-    qsort(header_list_, list_size, sizeof(UploadItem), cmp);
+    qsort(header_list_, list_size, sizeof(UploadItem),
+          reinterpret_cast<int (*)(const void *a, const void *b)>(cmp));
 
     header_list_ += start_index;
 }
