@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 2006 Tetsuya Kimata <kimata@acapulco.dyndns.org>
+ * Copyright (C) 2007 Tetsuya Kimata <kimata@acapulco.dyndns.org>
  *
  * All rights reserved.
  *
@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemManager.cpp 2604 2007-10-26 13:58:10Z svn $
+ * $Id: UploadItemManager.cpp 2756 2007-12-11 10:57:59Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -48,7 +48,7 @@
 
 using namespace std;
 
-SOURCE_INFO_ADD("$Id: UploadItemManager.cpp 2604 2007-10-26 13:58:10Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItemManager.cpp 2756 2007-12-11 10:57:59Z svn $");
 
 #ifdef DEBUG
 #include <iostream>
@@ -262,35 +262,32 @@ apr_size_t UploadItemManager::add_impl(apr_pool_t *pool,
     apr_size_t item_index;
     UploadItem *uitem;
     apr_time_t current_time;
-    WriteLocker manager_locker(lock_);
 
-    REQUIRE_MANAGER_LOCKED();
+    remove_old_files(pool, file_size);
 
-    create_item_id_index(&item_id, &item_index);
-    current_time = apr_time_now();
+    {
+        WriteLocker manager_locker(lock_);
 
-    uitem = UploadItem::get_instance
-        (pool, item_id, item_index, file_size, current_time, ip_address,
-         file_name, file_mime, file_digest, remove_pass, download_pass,
-         comment, code_pat);
+        REQUIRE_MANAGER_LOCKED();
 
-    // 古いアイテムを削除
-    while (item_list_->is_full(file_size)) {
-        UploadItem *old_uitem;
+        create_item_id_index(&item_id, &item_index);
+        current_time = apr_time_now();
 
-        old_uitem = AS_UITEM(item_list_->get_by_index(item_list_->size()-1));
+        uitem = UploadItem::get_instance
+            (pool, item_id, item_index, file_size, current_time, ip_address,
+             file_name, file_mime, file_digest, remove_pass, download_pass,
+             comment, code_pat);
 
-        item_writer_.remove(pool, old_uitem);
-        thumbnail_list_->remove(old_uitem->get_id());
-        item_list_->remove(old_uitem->get_id());
+        if (item_writer_.write(uitem, file_temp_path)) {
+            thumbnail_list_->add(uitem->get_id());
+        }
+        item_list_->add(uitem);
+
+        *mtime_ = current_time;
     }
 
-    if (item_writer_.write(uitem, file_temp_path)) {
-        thumbnail_list_->add(uitem->get_id());
-    }
-    item_list_->add(uitem);
-
-    *mtime_ = current_time;
+    // 2 回に分けてロックしているので，最悪の場合を考えると必要
+    remove_old_files(pool, 0);
 
     return item_id;
 }
@@ -309,6 +306,25 @@ void UploadItemManager::remove_impl(apr_pool_t *pool, UploadItem *uitem)
     *mtime_ = apr_time_now();
 }
 
+void UploadItemManager::remove_old_files(apr_pool_t *pool,
+                                         apr_uint64_t file_size)
+{
+    WriteLocker manager_locker(lock_);
+
+    REQUIRE_MANAGER_LOCKED();
+
+    // 古いアイテムを削除
+    while (item_list_->is_full(file_size)) {
+        UploadItem *old_uitem;
+
+        old_uitem = AS_UITEM(item_list_->get_by_index(item_list_->size()-1));
+
+        item_writer_.remove(pool, old_uitem);
+        thumbnail_list_->remove(old_uitem->get_id());
+        item_list_->remove(old_uitem->get_id());
+    }
+}
+
 void UploadItemManager::create_item_id_index(apr_size_t *item_id,
                                              apr_size_t *item_index) const
 {
