@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: cgi_uploader.cpp 2568 2007-10-11 15:47:19Z svn $
+ * $Id: cgi_uploader.cpp 2716 2007-12-01 12:34:40Z svn $
  *****************************************************************************/
 
 /**
@@ -30,6 +30,10 @@
  * @brief CGI のエントリ関数群．
  */
 
+#ifndef UPLOADER_TYPE_CGI
+#define UPLOADER_TYPE_CGI
+#endif
+
 #include "Environment.h"
 
 #include <iostream>
@@ -49,7 +53,7 @@
 #define TEMPLATE_INSTANTIATION
 #include "uploader_handler.cpp"
 
-SOURCE_INFO_ADD("$Id: cgi_uploader.cpp 2568 2007-10-11 15:47:19Z svn $");
+SOURCE_INFO_ADD("$Id: cgi_uploader.cpp 2716 2007-12-01 12:34:40Z svn $");
 
 // strace したときに，表示処理の範囲を見分けるための印
 #if defined(DEBUG) && defined(__linux__)
