@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 2006 Tetsuya Kimata <kimata@acapulco.dyndns.org>
+ * Copyright (C) 2007 Tetsuya Kimata <kimata@acapulco.dyndns.org>
  *
  * All rights reserved.
  *
@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: uploader_handler.cpp 2714 2007-12-01 12:27:48Z svn $
+ * $Id: uploader_handler.cpp 2756 2007-12-11 10:57:59Z svn $
  *****************************************************************************/
 
 #ifndef TEMPLATE_INSTANTIATION
@@ -57,16 +57,19 @@
 #include "SourceInfo.h"
 
 #ifndef TEMPLATE_INSTANTIATION
-SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2714 2007-12-01 12:27:48Z svn $");
+SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2756 2007-12-11 10:57:59Z svn $");
 #endif
 
-#define IS_REQUEST_COMMAND(arg, command)                        \
-    ((strncmp(arg, command, LITERAL_STRLEN(command)) == 0) &&    \
-     (arg[LITERAL_STRLEN(command)] == '\0'))
+#define STR_CMP_LITERAL(arg, literal)                            \
+    ((strncmp(arg, literal, LITERAL_STRLEN(literal)) == 0) &&    \
+     (arg[LITERAL_STRLEN(literal)] == '\0'))
+#define STR_CMP_LITERAL_START_WITH(arg, literal)            \
+    (strncmp(arg, literal, LITERAL_STRLEN(literal)) == 0)
 
 static const apr_size_t PER_PAGE_THUMBNAIL_NUMBER = 100;
 
 static const char BASE_URL_VAR_NAME[]           = "BASE_URL";
+static const char COMMAND_VAR_NAME[]            = "COMMAND";
 static const char ITEM_VAR_NAME[]               = "ITEM";
 static const char ITEM_LIST_VAR_NAME[]          = "ITEM_LIST";
 static const char THUMBNAIL_LIST_VAR_NAME[]     = "THUMBNAIL_LIST";
@@ -79,6 +82,7 @@ static const char CURRENT_PAGE_VAR_NAME[]       = "CURRENT_PAGE";
 static const char ERROR_MESSAGE_VAR_NAME[]      = "ERROR_MESSAGE";
 
 static const char INDEX_COMMAND[]               = "index";
+static const char SORT_COMMAND[]                = "sort_";
 static const char INFO_COMMAND[]                = "info";
 static const char THUMBNAIL_COMMAND[]           = "thumbnail";
 static const char THUMBNAIL_VIEW_COMMAND[]      = "view";
@@ -95,6 +99,23 @@ static const char UPLOAD_ID_ARG[]               = "upload_id";
 
 static const char PAGE_CONTENT_TYPE[]           = "text/html; charset=EUC-JP";
 
+UploadItem::CompareFunc get_sort_func(const char *sort_type)
+{
+    if (STR_CMP_LITERAL(sort_type, "file_name")) {
+        return UploadItem::cmp_file_name;
+    } else if (STR_CMP_LITERAL(sort_type, "file_size")) {
+        return UploadItem::cmp_file_size;
+    } else if (STR_CMP_LITERAL(sort_type, "file_mime")) {
+        return UploadItem::cmp_file_mime;
+    } else if (STR_CMP_LITERAL(sort_type, "comment")) {
+        return UploadItem::cmp_comment;
+    } else if (STR_CMP_LITERAL(sort_type, "download_count")) {
+        return UploadItem::cmp_download_count;
+    } else {
+        return NULL;
+    }
+}
+
 template<class Response>
 static int redirect(typename Response::Handle *r, typename Response::Writer& o,
                     const char *url)
@@ -245,6 +266,91 @@ static int index(typename Response::Handle *r, UploaderConfig *config,
         TemplateVariableCreator var_creator(r->pool, tmpl->get_ids());
 
         var_creator.create(BASE_URL_VAR_NAME, config->base_url);
+        var_creator.create(COMMAND_VAR_NAME, INDEX_COMMAND);
+
+        var_creator.create
+            (ITEM_LIST_VAR_NAME,
+             tmpl->get_item_var_creator()->create(r->pool, &item_iter));
+
+        var_creator.create(MAX_FILE_SIZE_VAR_NAME,
+                           size_str(r->pool, config->file_size_limit));
+        var_creator.create
+            (TOTAL_FILE_SIZE_VAR_NAME,
+             size_str(r->pool,
+                      config->get_item_list()->get_total_file_size()));
+        var_creator.create(TOTAL_FILE_NUMBER_VAR_NAME,
+                           config->get_item_list()->size());
+        var_creator.create(PAGE_COUNT_VAR_NAME, page_count);
+
+        var_creator.create(CURRENT_PAGE_VAR_NAME, page_no);
+
+        TemplateExecutor<typename Response::Writer> executor(r->pool, o);
+
+        executor.exec(tmpl->get_node_tree(), var_creator.get_variables(),
+                      tmpl->get_key_count());
+    } catch(const char *message) {
+        return error_with_log<Response>(r, o, config, message);
+    }
+    o.finish();
+
+    return OK;
+}
+
+template<class Response>
+static int sort(typename Response::Handle *r, UploaderConfig *config,
+                const char *arg, const char *sort_type)
+{
+    apr_size_t page_no;
+    apr_size_t page_count;
+    UploadItem::CompareFunc cmp_func;
+
+    cmp_func = get_sort_func(sort_type);
+    if (cmp_func == NULL) {
+        std::cout << "sort_type: " << sort_type << std::endl;
+        std::cerr << "sort_type: " << sort_type << std::endl;
+        return index<Response>(r, config, arg);
+    }
+
+    Response::set_content_type(r, PAGE_CONTENT_TYPE);
+#ifndef DEBUG
+    Response::set_modified_time(r, config->get_item_manager()->get_mtime());
+    Response::set_last_modified(r, config->get_item_manager()->get_mtime());
+
+    int status;
+    if ((status = Response::is_meets_condition(r)) != OK) {
+        return status;
+    }
+#endif
+    Response::header_end(r);
+
+    if (Response::is_head_method(r)) {
+        return OK;
+    }
+
+    get_page_count(config->get_item_list()->size(),
+                   config->per_page_item_number, &page_count);
+    get_page(r->pool, arg, page_count, &page_no);
+
+    typename Response::Writer o(r);
+    try {
+        UploaderTemplate *tmpl;
+        apr_size_t start_index;
+        apr_size_t end_index;
+
+        end_index = page_no * config->per_page_item_number;
+        start_index = end_index - config->per_page_item_number;
+
+        UploadItemIterator item_iter(r->pool, config->get_item_manager(),
+                                     start_index, end_index, cmp_func);
+
+        tmpl = config->get_template(UploaderConfig::INDEX_VIEW);
+
+        TemplateVariableCreator var_creator(r->pool, tmpl->get_ids());
+
+        var_creator.create(BASE_URL_VAR_NAME, config->base_url);
+        var_creator.create
+            (COMMAND_VAR_NAME,
+             apr_pstrcat(r->pool, SORT_COMMAND, sort_type, NULL));
 
         var_creator.create
             (ITEM_LIST_VAR_NAME,
@@ -441,7 +547,7 @@ static int thumbnail(typename Response::Handle *r, UploaderConfig *config,
     thumbnail_arg = arg;
     thumbnail_command = get_word(r->pool, &thumbnail_arg, ARG_SEPARATE_STR[0]);
 
-    if (IS_REQUEST_COMMAND(thumbnail_command, THUMBNAIL_VIEW_COMMAND)) {
+    if (STR_CMP_LITERAL(thumbnail_command, THUMBNAIL_VIEW_COMMAND)) {
         return thumbnail_view<Response>(r, config, thumbnail_arg);
     }
     Response::header_end(r);
@@ -1119,11 +1225,11 @@ static int admin(typename Response::Handle *r, UploaderConfig *config,
     admin_arg = arg;
     admin_command = get_word(r->pool, &admin_arg, ARG_SEPARATE_STR[0]);
 
-    if (IS_REQUEST_COMMAND(admin_command, DOWNLOAD_COMMAND)) {
+    if (STR_CMP_LITERAL(admin_command, DOWNLOAD_COMMAND)) {
         return download<Response>(r, config, admin_arg, true);
     } else if (strcmp(admin_command, REMOVE_COMMAND) == 0) {
         return remove<Response>(r, config, admin_arg, true);
-    } else if (IS_REQUEST_COMMAND(admin_command, ADMIN_INFO_COMMAND)) {
+    } else if (STR_CMP_LITERAL(admin_command, ADMIN_INFO_COMMAND)) {
         return admin_info<Response>(r, config);
     }
 
@@ -1207,25 +1313,28 @@ int uploader_command_handler(typename Response::Handle *r, UploaderConfig *confi
 
     command = get_word(r->pool, &arg, ARG_SEPARATE_STR[0]);
 
-    if ((*command == '\0') || IS_REQUEST_COMMAND(command, INDEX_COMMAND)) {
+    if ((*command == '\0') || STR_CMP_LITERAL(command, INDEX_COMMAND)) {
         return index<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, INFO_COMMAND)) {
+    } else if (STR_CMP_LITERAL_START_WITH(command, SORT_COMMAND)) {
+        return sort<Response>(r, config, arg,
+                              command + LITERAL_STRLEN(SORT_COMMAND));
+    } else if (STR_CMP_LITERAL(command, INFO_COMMAND)) {
         return info<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, THUMBNAIL_COMMAND)) {
+    } else if (STR_CMP_LITERAL(command, THUMBNAIL_COMMAND)) {
         return thumbnail<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, UPLOAD_COMMAND)) {
+    } else if (STR_CMP_LITERAL(command, UPLOAD_COMMAND)) {
         return upload<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, PROGRESS_DATA_COMMAND)) {
+    } else if (STR_CMP_LITERAL(command, PROGRESS_DATA_COMMAND)) {
         return progress_data<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, DOWNLOAD_COMMAND)) {
+    } else if (STR_CMP_LITERAL(command, DOWNLOAD_COMMAND)) {
         return download<Response>(r, config, arg);
     } else if (strcmp(command, REMOVE_COMMAND) == 0) {
         return remove<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, RSS_COMMAND)) {
+    } else if (STR_CMP_LITERAL(command, RSS_COMMAND)) {
         return rss<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, MAIL_COMMAND)) {
+    } else if (STR_CMP_LITERAL(command, MAIL_COMMAND)) {
         return mail_upload<Response>(r, config, arg);
-    } else if (IS_REQUEST_COMMAND(command, ADMIN_COMMAND)) {
+    } else if (STR_CMP_LITERAL(command, ADMIN_COMMAND)) {
         return admin<Response>(r, config, arg);
     } else {
         return error<Response>(r, config, MESSAGE_HANDLER_COMMAND_INVALID);
